import numpy as np
import time


t = 100.0  # t/U
lam = 100.0  # lambda / U
a = 1.0 # lattice parameter

N = 100000.0 # lattice points in each direction

alphas = np.array([0.8,0.9,1.0,1.1,1.5])  # check for several alpha = U_updown / U


mu = 0.9
kappa = 0.9
start = time.time()

cosgammak=singammak=cos2gammak=sin2gammak=sumcos=sumsin=E_p=E_m=F=A=B=C=G1sq=G2sq=Gsum=R=K=H = 0
eps_p=eps_m=eps_0=Deleps_p=Deleps_m=0


deltak = 2*np.pi/(a*N)                      # step size in k-space
kx = np.arange(-np.pi/a, np.pi/a, deltak)
ky = np.arange(-np.pi/a, np.pi/a, deltak)
kx = np.delete(kx,np.argwhere(kx==0))
ky = np.delete(ky,np.argwhere(ky==0))


step = 0.05                                 # step size in n-space
n = np.arange(0, 1+step, step)
n1,n2,n3,n4 = np.meshgrid(n,n,n,n)
sum_mesh = n1+n2+n3+n4
sub_mesh = np.argwhere(sum_mesh==kappa)     # mesh of combinations of n satisfying sum n = kappa

comboNumber = sub_mesh.shape[0]

aNumber = alphas.shape[0]

def main():
    indicesFri = []
    counter2=0
    lastValue = -1
    for alpha in alphas:
        Fri = np.zeros(sub_mesh.shape[0])
        counter=0
        for i,j,k,l in sub_mesh:
            Fri[counter] = freeEnergy(n1[i,j,k,l],n2[i,j,k,l],n3[i,j,k,l],n4[i,j,k,l],alpha)    # sum term
            Fri[counter] += constant(n1[i,j,k,l],n2[i,j,k,l],n3[i,j,k,l],n4[i,j,k,l],alpha)     # constant term
            counter +=1
            percentage = int(100.0*(counter+comboNumber*counter2)/(comboNumber*aNumber))
            if percentage%10==0 and lastValue != percentage:
                if percentage!=0:
                    now = time.time()
                    left = (now-start)*(100.0/percentage - 1.0)
                    hours = np.floor(left/(60*60))
                    minutes = np.floor((left-hours*60*60)/60)
                    sec = left-hours*60*60-minutes*60
                    print('%i percent done. %i hours %i minutes %i seconds left' % (int(percentage), int(hours), int(minutes),int(sec)))
                lastValue = percentage
        minimum2 = np.min(Fri)
        indicesFri.append(np.argwhere(Fri==minimum2)[:,0])
        counter2 += 1

    indices=[]

    for i in indicesFri:
        c = []
        for j in range(0,i.shape[0]):
            c.append(sub_mesh[i[j]])
        indices.append(c)
    
    print('\n')
    print('N = %i' % N)
    print('Step in n-mesh: %f' % float(step))
    
    ## Print the n combinations minimizing F:
    
    for j in range(0,aNumber):
        print('\n')
        print('alpha = %f' % float(alphas[j]))
        print('[n1, n2, n3, n4] =')
        ind = np.asarray(indices[j])
        for i in range(0,indicesFri[j].shape[0]):
             print(n1[ind[i,0],ind[i,1],ind[i,2],ind[i,3]], n2[ind[i,0],ind[i,1],ind[i,2],ind[i,3]], n3[ind[i,0],ind[i,1],ind[i,2],ind[i,3]], n4[ind[i,0],ind[i,1],ind[i,2],ind[i,3]])
         

cosgammak = np.sin(kx*a)/np.sqrt(np.sin(kx*a)**2+np.sin(ky*a)**2)
singammak = -np.sin(ky*a)/np.sqrt(np.sin(kx*a)**2+np.sin(ky*a)**2)
cos2gammak = cosgammak**2 - singammak**2
sin2gammak = 2*cosgammak*singammak
eps_p = - 2*t*(np.cos(kx*a)+np.cos(ky*a))+ 2*lam*np.sqrt(np.sin(kx*a)**2+np.sin(ky*a)**2)
eps_m = - 2*t*(np.cos(kx*a)+np.cos(ky*a))- 2*lam*np.sqrt(np.sin(kx*a)**2+np.sin(ky*a)**2)
eps_0 = - 2*np.sqrt(2)*np.sqrt(2*t**2+lam**2)
Deleps_p = eps_p-eps_0
Deleps_m = eps_m-eps_0

def freeEnergy(m1,m2,m3,m4,alpha):
    Energy1 = 0
    Energy2 = 0
    sumcos = (m1-m2-m3+m4)*cosgammak/np.sqrt(2) - (m1+m2-m3-m4)*singammak/np.sqrt(2)
    sumsin = (m1+m2-m3-m4)*cosgammak/np.sqrt(2) + (m1-m2-m3+m4)*singammak/np.sqrt(2)

    E_p = Deleps_p + mu*(1+alpha/2) - (alpha/2)*sumcos
    E_m = Deleps_m + mu*(1+alpha/2) - (alpha/2)*sumcos

    F = alpha*sumsin/2.0 # *i
    A = (np.sqrt(m1*m3)+np.sqrt(m2*m4))/2
    B = sin2gammak*(np.sqrt(m1*m3)-np.sqrt(m2*m4))/2 
    C = cos2gammak*(np.sqrt(m1*m3)-np.sqrt(m2*m4))/2
    G1sq = (A - B)**2 + C**2
    G2sq = (A + B)**2 + C**2
    R = ((E_p+E_m)**2 - 4*G2sq)*(E_p-E_m)**2
    K = E_p**2+E_m**2 + F**2 - G1sq - 2.0*G2sq
    Energy1 = np.sqrt(2*(K+np.sqrt(R)) + 2*abs(G1sq-F**2))/4.0      # Energy of + branch, summed over sigma = +,-
    Energy2 = np.sqrt(2*(K-np.sqrt(R)) + 2*abs(G1sq-F**2))/4.0      # Energy of - branch, summed over sigma = +,-
    return np.sum(Energy1+Energy2)                                  # Summing over branches and k vectors
    
gammas = np.array([-np.pi/4, -3*np.pi/4, 3*np.pi/4, np.pi/4])

def constant(m1,m2,m3,m4,alpha):
    ms = np.array([m1,m2,m3,m4])
    realSum = 2*(m1*(m2+m3+m4)+m2*(m3+m4)+m3*m4)
    #realSum = 0
    compSum = 0
    for i in range(4):
        for j in range(i):
            #realSum += 2*ms[i]*ms[j]
            compSum += ms[i]*ms[j]*2*np.cos(gammas[j]-gammas[i])
    return (N**2/8.0) * ( 2.0*realSum + 2.0*alpha*compSum) 

if __name__=="__main__":
   main()
